% This routine can be used to check the conservation of the metabolite pools
% during an experiment.
%
% function [ pools, names ] = checkPools( modelInfo, x, plots, t )
%
% Mandatory input parameters:
%   modelInfo    - modelInfo structure
% Optional input parameters:
%   x            - Solution vector for the simulation (to obtain concs)
%   plots        - 1 if the pools are to be plotted
%   t            - time vector, if results are to be plotted against time
%
% Output parameters:
%   pools        - Numerical values of the pools at time steps
%   names        - Names of the variables in the pools vector

function [ pools, names ] = checkPools( modelInfo, x, plots, t )
% Copyright (C) 2008 Medical College of Wisconsin.
% All rights reserved.
% 
% This software is distributed freely for academic and non-profit use. It
% should not be distributed or used for any commercial purpose without
% written permission from the Medical College of Wisconsin. For details
% please see User License Agreement.

if nargin < 3
    plots = 0;
end

tC = modelInfo.matrices.conservationMatrix;
tC( find( tC < 1000 * eps ) ) = 0;

[nx, ny] = size( tC );

if nargin > 1
    pools = ( tC * x(:, modelInfo.matrices.conservationVars).' ).';
    diffpools = diff( pools );
end

for a = 1 : nx
    names{a} = modelInfo.SVarList( modelInfo.matrices.conservationVars( find( abs( tC(a,:) ) > 1000*eps ) ) );
    legends{a} = num2str(a);

    if nargin > 1
        disp( sprintf( 'Pool %d (Avg value: %.4g, Max change: %.4g)', a, mean( pools(:,a) ), max(diffpools(:,a))) );
    else
        disp( sprintf( 'Pool %d', a ) );
    end
    for b = 1 : length( names{a} )
        disp( sprintf( '      %s', names{a}{b} ) );
    end
  
end 

if plots == 1
    if nargin > 3
        plot( t, pools );
    else
        plot( pools );            
    end            
   title( 'Total pools' );
    legend( legends );
end

if nargin > 1
    disp( sprintf( 'Max pool change: %.6g', max( max( diff( pools ) ) ) ) );
end

if nargout < 1
    pools = [];
    names = [];
end
