% Copyright (C) 2008 Medical College of Wisconsin.
% All rights reserved.
% 
% This software is distributed freely for academic and non-profit use. It
% should not be distributed or used for any commercial purpose without
% written permission from the Medical College of Wisconsin. For details
% please see User License Agreement.

clear all;


%% Load the model info file when it already exists
% load dXdTBose.mat

%% Setting paths
path(path,'..\Warnings');
path(path,'..\BiochemicalReactions');
path(path,'..\Transporters');
path(path,'..\Databases');
path(path,'..\BuilderFiles');
path(path,'..\ExperimentalData');
path(path,'..\Examples');

%% Building model:
modelInfo = BuildDXDT('Example3_Bose.bsl','dXdTBose.m');
save dXdTBose.mat

%% Set desired temperature
T = 37;

%% Set proton buffer sizes
BX(1) = 0.02;       % matrix
BX(2) = 1000000;     % cytoplasm
BX(3) = 1000000;     % intermembrane space

%% Set proton buffer binding constant(s)
K_BX(1) = 1e-7;
K_BX(2) = 10^(-7.1);
K_BX(3) = 10^(-7.1);

%% incubation time
t_ib = 60;

%% Setting physical parameters and fixed pools for computing the graphs
W_m = 0.664*1.09;   % mitochondrial water space (ml water per ml mito) [VB2002]
W_x = 0.9*W_m;      % Matrix water space, (l mito water) (l mito)^{-1}
W_i = 0.1*W_m;      % IM water space, (l IM water) (l mito)^{-1} 
W_c = 800;          % buffer, (l buffer water) (l mito)^{-1}
rho_m = 3.6697e-6;  % (l mito) (mg protein)^{-1}

Ctot   = 2.70e-3;               % M; total cytoC
Qtot   = 1.35e-3;               % M; total Q + QH2
NADtot = 2.97e-3;               % M; total NAD+NADH
Atot   = 10e-3;                 % M; total Adenosine

%% Make sure all the variables except the membrane potentials cannot become negative
varlist = [ 1 : length( modelInfo.SVarList ) ];
varlist( [ modelInfo.SVarID.DPsi_im_to_matrix,  modelInfo.SVarID.DPsi_cytoplasm_to_im ] ) = [];
options = odeset('MaxStep',5e-1,'NonNegative', varlist, 'MaxOrder', 2 ); %5e-2

%% Load parameters and initial conditions for in vitro simulations
x0 = TCA_x0_vitro( modelInfo );
params = TCA_params( modelInfo );

%% Validate parameters
validateParameters( params, modelInfo, 0 );
validateParameters( x0, modelInfo, 1 );

% No hexokinase activity
params = setParameter( params, modelInfo.ParID, 'x_HK', 0 );

%% Add species for incubation
x0 = setParameter( x0, modelInfo.SVarID, 'K_matrix', 0.125 );      % Bose et al., 2003

x0 = setParameter( x0, modelInfo.SVarID, 'H_cytoplasm', 10^(-7.1) );   % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'H_im', 10^(-7.1) );          % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'M_cytoplasm', 5e-3 );        % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'M_im', 5e-3 );               % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'K_cytoplasm', 150e-3 );      % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'K_im', 150e-3 );             % Bose et al., 2003

x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', 0.001e-3 );   % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'malate_cytoplasm',  5e-3 );  % Bose et al., 2003
x0 = setParameter( x0, modelInfo.SVarID, 'glutamate_cytoplasm', 5e-3 );  % Bose et al., 2003

%% Incubate
disp( 'Incubating...' );
[t,x] = ode15s(@dXdTBose, [0 t_ib], x0, options, T, BX, K_BX, params  );
xIncubated = x(end,:);

%% Loading state 2 experimental data curves (from Bose et al., 2003)
load Pi_exp_r; Pi_exp_r = Pi_exp_r/1e3;
load d_Psi_exp_r;
load NADH_exp_r;

%% Loading state 3 experimental data curves (from Bose et al., 2003)
load Pi_exp_a; Pi_exp_a = Pi_exp_a/1e3;
load d_Psi_exp_a;
load NADH_exp_a;
load JO2_exp_a;

%% State 2 Cytochrome C data--percent reduced (from Bose et al., 2003)
CPR_exp_r(1) = 0.092;
CPR_exp_r(2) = 0.105;

%% State 3 Cytochrome C data--percent reduced (from Bose et al., 2003)
CPR_exp_a(1) = 0.084;
CPR_exp_a(2) = 0.171;

%% Compute curves for state 2
x0 = xIncubated;

for i = 1:1:21
    i
    % Adding Pi into the buffer 
    Pi_r(i) = (i-1)*0.1/1e3;
    x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', Pi_r(i) );   % Bose et al., 2003
    if x0( modelInfo.SVarID.Pi_cytoplasm ) < 0.001e-3
        x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', 0.001e-3 );   % Bose et al., 2003
    end

    [t,x] = ode15s(@dXdTBose, [0 t_ib], x0, options, T, BX, K_BX, params );
   
    H_x_PI_r(i) = x(end,modelInfo.SVarID.H_matrix);
    NADH_x_PI_r(i) = x(end,modelInfo.SVarID.NADH_matrix);
    Cred_i_PI_r(i) = x(end,modelInfo.SVarID.cytocred_im);
    dPsi_PI_r(i) = x(end,modelInfo.SVarID.DPsi_im_to_matrix);       
    ATP_PI_r(i) = x(end,modelInfo.SVarID.ATP_matrix);
    
    % Compute oxygen consumption rates
    [junk,J] = dXdTBose( t(end), x(end,:).', T, BX, K_BX, params );
    JO2_r(i) = ( J( modelInfo.FluxID.ETC4_im_to_matrix ) / 2 ) * 1.8957e5; %ATP_matrix, DPsi

end
for i = 22:1:41
    i
    % Adding Pi into the buffer
    Pi_r(i) = ((i-21)*0.4 + 2)/1e3;
    x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', Pi_r(i) );   % Bose et al., 2003
    if x0( modelInfo.SVarID.Pi_cytoplasm ) < 0.001e-3
        x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', 0.001e-3 );   % Bose et al., 2003
    end

   [t,x] = ode15s(@dXdTBose, [0 t_ib], x0, options, T, BX, K_BX, params );
   
    H_x_PI_r(i) = x(end,modelInfo.SVarID.H_matrix);
    NADH_x_PI_r(i) = x(end,modelInfo.SVarID.NADH_matrix);
    Cred_i_PI_r(i) = x(end,modelInfo.SVarID.cytocred_im);
    dPsi_PI_r(i) = x(end,modelInfo.SVarID.DPsi_im_to_matrix);       
    ATP_PI_r(i) = x(end,modelInfo.SVarID.ATP_matrix);
    
    % Compute oxygen consumption rates
    [junk,J] = dXdTBose( t(end), x(end,:).', T, BX, K_BX, params  );
    JO2_r(i) = ( J( modelInfo.FluxID.ETC4_im_to_matrix ) / 2 ) * 1.8957e5; %ATP_matrix, DPsi
end

%% Compute curves for state 3
x0 = xIncubated;

for i = 1:1:21
    i
    % Adding ADP into the buffer
    x0 = setParameter( x0, modelInfo.SVarID, 'ADP_cytoplasm', 1.3e-3 );             % Bose et al., 2003
    
    [t,x] = ode15s(@dXdTBose, [0 60], x0, options, T, BX, K_BX, params );
    x01 = x(end,:);
    
    % Adding Pi into the buffer 
    Pi_a(i) = (i-1)*0.1/1e3;
    x01 = setParameter( x01, modelInfo.SVarID, 'Pi_cytoplasm', Pi_a(i) );   % Bose et al., 2003
    if x01( modelInfo.SVarID.Pi_cytoplasm ) < 0.001e-3
        x01 = setParameter( x01, modelInfo.SVarID, 'Pi_cytoplasm', 0.001e-3 );   % Bose et al., 2003
    end
    [t,x] = ode15s(@dXdTBose, [0 30], x01, options, T, BX, K_BX, params );
   
    H_x_PI_a(i) = x(end,modelInfo.SVarID.H_matrix);
    NADH_x_PI_a(i) = x(end,modelInfo.SVarID.NADH_matrix);
    Cred_i_PI_a(i) = x(end,modelInfo.SVarID.cytocred_im);
    dPsi_PI_a(i) = x(end,modelInfo.SVarID.DPsi_im_to_matrix);       
    ATP_PI_a(i) = x(end,modelInfo.SVarID.ATP_matrix);
    
    % Compute oxygen consumption rates
    [junk,J] = dXdTBose( t(end), x(end,:).', T, BX, K_BX, params );
    JO2_a(i) = ( J( modelInfo.FluxID.ETC4_im_to_matrix ) / 2 ) * 1.8957e5; %ATP_matrix, DPsi
end
for i = 22:1:41
    i
    % Adding ADP into the buffer
    x0 = setParameter( x0, modelInfo.SVarID, 'ADP_cytoplasm', 1.3e-3 );             % Bose et al., 2003
    
    [t,x] = ode15s(@dXdTBose, [0 60], x0, options, T, BX, K_BX, params );
    x01 = x(end,:);
    
    % Adding Pi into the buffer 
    Pi_a(i) = ((i-21)*0.4+2)/1e3;
    x01 = setParameter( x01, modelInfo.SVarID, 'Pi_cytoplasm', Pi_a(i) );   % Bose et al., 2003
    if x01( modelInfo.SVarID.Pi_cytoplasm ) < 0.001e-3
        x01 = setParameter( x01, modelInfo.SVarID, 'Pi_cytoplasm', 0.001e-3 );   % Bose et al., 2003
    end
    [t,x] = ode15s(@dXdTBose, [0 30], x01, options, T, BX, K_BX, params );
   
    H_x_PI_a(i) = x(end,modelInfo.SVarID.H_matrix);
    NADH_x_PI_a(i) = x(end,modelInfo.SVarID.NADH_matrix);
    Cred_i_PI_a(i) = x(end,modelInfo.SVarID.cytocred_im);
    dPsi_PI_a(i) = x(end,modelInfo.SVarID.DPsi_im_to_matrix);       
    ATP_PI_a(i) = x(end,modelInfo.SVarID.ATP_matrix);
    
    % Compute oxygen consumption rates
    [junk,J] = dXdTBose( t(end), x(end,:).', T, BX, K_BX, params );
    JO2_a(i) = ( J( modelInfo.FluxID.ETC4_im_to_matrix ) / 2 ) * 1.8957e5; %ATP_matrix, DPsi
end

%% Plotting the results
figure(4); clf;

% [NADH]
subplot(3,2,1);
box on; hold on;
plot(Pi_r*1e3,NADH_x_PI_r/NADtot,'--k','LineWidth',2);
plot(Pi_exp_r*1e3,NADH_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,NADH_x_PI_a/NADtot,'-k','LineWidth',2);
plot(Pi_exp_a*1e3,NADH_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',14);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:0.2:1);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)');
ylabel('NADH (Normalized)');
xlim([0 10]);
ylim([0 1.0]);
text((10-0)*0.05,(1-0)*0.9,'A','FontSize',24);
text((10-0)*0.35,(1-0)*0.88,'[ADP]_{buffer} = 0 mM','FontSize',12);
text((10-0)*0.35,(1-0)*0.44,'[ADP]_{buffer} = 1.3 mM','FontSize',12);

% [MVO2]
subplot(3,2,2);
box on; hold on;
plot(Pi_r*1e3,JO2_r,'--k','LineWidth',2);
plot([0 3],[30.8 40],'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,JO2_a,'-k','LineWidth',2);
plot(Pi_exp_a*1e3,JO2_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',14);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:50:350);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)')
ylabel('MV_{O_2} (mol O_2 min^{-1} (mol Cyto A)^{-1})');
xlim([0 10]);
ylim([0 350]);
text((10-0)*0.05,(350-0)*0.9,'B','FontSize',24);
text((10-0)*0.35,(350-0)*0.23,'[ADP]_{buffer} = 0 mM','FontSize',12);
text((10-0)*0.35,(350-0)*0.67,'[ADP]_{buffer} = 1.3 mM','FontSize',12);

% [cytoC]
subplot(3,2,3);
box on;hold on;
plot(Pi_r*1e3,Cred_i_PI_r/Ctot,'--k','LineWidth',2);
plot([0 3],CPR_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,Cred_i_PI_a/Ctot,'-k','LineWidth',2);
plot([0 3],CPR_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',14);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:0.05:0.25);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)');
ylabel('CytoC^{2+} (Normalized)');
xlim([0 10]);
ylim([0 0.25]);
text((10-0)*0.05,(0.25-0)*0.9,'C','FontSize',24);
text((10-0)*0.35,(0.25-0)*0.53,'[ADP]_{buffer} = 0 mM','FontSize',12);
text((10-0)*0.35,(0.25-0)*0.8,'[ADP]_{buffer} = 1.3 mM','FontSize',12);

% [pH_x]
subplot(3,2,4);
box on;hold on;
p = plot(Pi_r*1e3,-log10(H_x_PI_r),'--k','LineWidth',2);
plot([0 3],[7.14 7.13],'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
p = plot(Pi_a*1e3,-log10(H_x_PI_a),'-k','LineWidth',2);
plot([0 3],[7.16 7.10],'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',14);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',6.5:0.25:7.5);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)');
ylabel('Matrix pH');
xlim([0 10]);
ylim([6.5 7.5]);
text((10-0)*0.05,(7.5-6.5)*0.9+6.5,'D','FontSize',24);
text((10-0)*0.35,(7.5-6.5)*0.5+6.5,'[ADP]_{buffer} = 0 mM','FontSize',12);
text((10-0)*0.35,(7.5-6.5)*0.7+6.5,'[ADP]_{buffer} = 1.3 mM','FontSize',12);

% [DeltaPsi]
subplot(3,2,5);
box on; hold on;
plot(Pi_r*1e3,dPsi_PI_r,'--k','LineWidth',2);
plot(Pi_exp_r*1e3,d_Psi_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,dPsi_PI_a,'-k','LineWidth',2);
plot(Pi_exp_a*1e3,d_Psi_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',14);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',130:20:210);
% set(gca,'XtickLabel',[]);
xlabel('Buffer [PI] (mM)');
ylabel('\Delta\Psi (mV)');
xlim([0 10]);
ylim([130 215]);
text((10-0)*0.05,(215-130)*0.9+130,'E','FontSize',24);
text((10-0)*0.35,(215-130)*0.90+130,'[ADP]_{buffer} = 0 mM','FontSize',12);
text((10-0)*0.35,(215-130)*0.62+130,'[ADP]_{buffer} = 1.3 mM','FontSize',12);

% [ATP-N]
subplot(3,2,6);
box on; hold on;
plot(Pi_r*1e3,ATP_PI_r/Atot,'--k','LineWidth',2);
plot(Pi_a*1e3,ATP_PI_a/Atot,'-k','LineWidth',2);
set(gca,'Fontsize',14);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:0.2:1);
% set(gca,'XtickLabel',[]);
xlabel('Buffer [PI] (mM)');
ylabel('Matrix ATP/A_{tot}');
xlim([0 10]);
ylim([0 1]);
text((10-0)*0.05,(1-0)*0.9,'F','FontSize',24);
text((10-0)*0.35,(1-0)*0.82,'[ADP]_{buffer} = 0 mM','FontSize',12);
text((10-0)*0.35,(1-0)*0.33,'[ADP]_{buffer} = 1.3 mM','FontSize',12);
