% Copyright (C) 2008 Medical College of Wisconsin.
% All rights reserved.
% 
% This software is distributed freely for academic and non-profit use. It
% should not be distributed or used for any commercial purpose without
% written permission from the Medical College of Wisconsin. For details
% please see User License Agreement.


clear all;


%% Load the model info file when it already exists
% load dXdTLaNoue.mat

%% Setting paths
path(path,'..\Warnings');
path(path,'..\BiochemicalReactions');
path(path,'..\Transporters');
path(path,'..\Databases');
path(path,'..\BuilderFiles');
path(path,'..\ExperimentalData');
path(path,'..\Examples');

%% Building model:
modelInfo = BuildDXDT('Example3_LaNoue.bsl','dXdTLaNoue.m');
save dXdTLaNoue.mat

%% Set the desired temperature
T = 28;

%% Set proton cytoplasm sizes
BX(1) = 0.02;       % matrix
BX(2) = 100000;     % cytoplasm
BX(3) = 100000;     % intermembrane space

%% Proton cytoplasm binding constant(s)
K_BX(1) = 1e-7;
K_BX(2) = 10^(-7.2);
K_BX(3) = 10^(-7.2);

%% incubation time
t_ib = 30;

%% Setting physical parameters for computing the graphs
W_m = 0.664*1.09;   % mitochondrial water space (ml water per ml mito) [VB2002]
W_x = 0.9*W_m;      % Matrix water space, (l mito water) (l mito)^{-1}
W_i = 0.1*W_m;      % IM water space, (l IM water) (l mito)^{-1} 
W_c = 80;           % cytoplasm, (l cytoplasm water) (l mito)^{-1}
rho_m = 3.6697e-6;  % (l mito) (mg protein)^{-1}

%% Set some solver options
opt = odeset( 'RelTol', 1e-6 );
opt = odeset( 'AbsTol', 1e-7 );

%% Make sure all the variables except the membrane potentials cannot become negative
varlist = [ 1 : length( modelInfo.SVarList ) ];
varlist( [ modelInfo.SVarID.DPsi_im_to_matrix,  modelInfo.SVarID.DPsi_cytoplasm_to_im ] ) = [];
options = odeset('MaxStep',4e-1,'NonNegative', varlist);

%% Load the parameters and initial conditions for in vitro simulations
x0 = TCA_x0_vitro( modelInfo );
params = TCA_params( modelInfo );

%% Validate parameters
validateParameters( params, modelInfo, 0 );
validateParameters( x0, modelInfo, 1 );

%% Disable hexokinase
params = setParameter( params, modelInfo.ParID, 'x_HK', 0 );

%% Add PI for incubation
x0 = setParameter( x0, modelInfo.SVarID, 'H_cytoplasm', 10^(-7.2) );   % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'M_cytoplasm', 5e-3 );        % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'K_cytoplasm', 150e-3 );      % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', 20e-3 );      % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'ADP_cytoplasm', 1e-9 );      % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'glucose_cytoplasm', 30e-3 ); % LaNoue et al., 1970

NADTotal = 2.97e-3;                                                    % Total NAD/NADH pool
x0 = setParameter( x0, modelInfo.SVarID, 'NAD_matrix' , NADTotal - 1.5e-3 );
x0 = setParameter( x0, modelInfo.SVarID, 'NADH_matrix', 1.5e-3 ); % LaNoue et al., 1970

%% Incubate
disp( 'Incubating...' );
t = [];
[t,y] = ode15s(@dXdTLaNoue, [0 t_ib], x0, options, T, BX, K_BX, params );
x1 = y;
t1 = t;

%% Add pyruvate and malate
x0 = y(end, :);
x0 = setParameter( x0, modelInfo.SVarID, 'pyruvate_cytoplasm' ,   2e-3 );
x0 = setParameter( x0, modelInfo.SVarID, 'malate_cytoplasm' ,    5e-3 );

%% Run the experiment for 500 seconds
disp( 'Doing the experiment...' );
[t,y] = ode15s(@dXdTLaNoue, [0 500], x0, options, T, BX, K_BX, params );
x = y;
torig = t;

%% Load experimental data to compare results with
load state4con.mat;
t_exp = state4con(:,1);

CT_exp = state4con(:,3);
AKG_exp = state4con(:,4);
SUC_exp = state4con(:,5);
MAL_exp = state4con(:,6);
FUM_exp = state4con(:,7);
PYR_uptake_exp = state4con(:,8);

% According to 2mM and 5mM initial concentration for Pyruvate and Malate in
% LaNoue et al's 1970 paper
PYR_exp = 2/5*MAL_exp(1) - PYR_uptake_exp; 

load fig7cit.mat;
t_exp = fig7cit(:,1);
CTtot_exp = fig7cit(:,2);
CTextm_exp = fig7cit(:,3);
CTmito_exp = fig7cit(:,5);

load fig3gluasp.mat;
t_exp_fig3 = fig3gluasp(:,1);
GLU_exp = fig3gluasp(:,4);
ASP_exp = fig3gluasp(:,5);

%% Stick the incubation and experiment together
x = [x1' x']';
t = [(t1-t_ib)' t']';


%% Compute simulated concentrations
CIT_mito  = ( x(:,modelInfo.SVarID.citrate_matrix)*W_x + x(:,modelInfo.SVarID.citrate_im)*W_i )*rho_m; 
CIT_extm  = x(:,modelInfo.SVarID.citrate_cytoplasm)*W_c*rho_m;
ICIT_mito = ( x(:,modelInfo.SVarID.isocitrate_matrix)*W_x + x(:,modelInfo.SVarID.isocitrate_im)*W_i )*rho_m; 
ICIT_extm = x(:,modelInfo.SVarID.isocitrate_cytoplasm)*W_c*rho_m;
CT_mito   = CIT_mito + ICIT_mito;
CT_extm   = CIT_extm + ICIT_extm;

AKG_mito = ( x(:,modelInfo.SVarID.ketoglutarate_matrix)*W_x + x(:,modelInfo.SVarID.ketoglutarate_im)*W_i )*rho_m; 
AKG_extm = x(:,modelInfo.SVarID.ketoglutarate_cytoplasm)*W_c*rho_m;

SUC_mito = ( x(:,modelInfo.SVarID.succinate_matrix)*W_x + x(:,modelInfo.SVarID.succinate_im)*W_i )*rho_m; 
SUC_extm = x(:,modelInfo.SVarID.succinate_cytoplasm)*W_c*rho_m;

MAL_mito = ( x(:,modelInfo.SVarID.malate_matrix)*W_x + x(:,modelInfo.SVarID.malate_im)*W_i )*rho_m; 
MAL_extm = x(:,modelInfo.SVarID.malate_cytoplasm)*W_c*rho_m;

FUM_mito = ( x(:,modelInfo.SVarID.fumarate_matrix)*W_x + x(:,modelInfo.SVarID.fumarate_im)*W_i )*rho_m; 
FUM_extm = x(:,modelInfo.SVarID.fumarate_cytoplasm)*W_c*rho_m; 

PYR_mito = ( x(:,modelInfo.SVarID.pyruvate_matrix)*W_x + x(:,modelInfo.SVarID.pyruvate_im)*W_i )*rho_m; 
PYR_extm = x(:,modelInfo.SVarID.pyruvate_cytoplasm)*W_c*rho_m;

OAA_mito = x(:,modelInfo.SVarID.oxaloacetate_matrix)*W_x *rho_m; 

SCOA_mito = x(:,modelInfo.SVarID.succinylcoA_matrix)*W_x *rho_m; 

%% Plot figures

figure(2); clf;

% [PYR]
subplot(3,2,1);
box on; hold on;
plot(t((length(t1)+1):end),(PYR_mito((length(t1)+1):end)+PYR_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, PYR_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',14);
set(gca,'Ytick',0:200:800);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[PYR]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 800]);
text((500-0)*0.05,(800-0)*0.9,'A','FontSize',24);

% [CIT]
subplot(3,2,2);
box on; hold on;
plot(t((length(t1)+1):end),(CT_mito((length(t1)+1):end)+CT_extm((length(t1)+1):end))*1e9,'k-',...
     t((length(t1)+1):end),CT_extm((length(t1)+1):end)*1e9,'k--','LineWidth',2.0);
plot(t_exp,CTtot_exp,'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
plot(t_exp,CTextm_exp,'ko','MarkerFaceColor', [1 1 1],'MarkerSize',12);
set(gca,'Fontsize',14);
set(gca,'Ytick',0:10:30);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[CIT]_{cytoplasm} and [CIT]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 30]);
text((500-0)*0.05,(30-0)*0.9,'B','FontSize',24);

% [AKG]
subplot(3,2,3);
box on; hold on;
plot(t((length(t1)+1):end),(AKG_mito((length(t1)+1):end)+AKG_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, AKG_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',14);
set(gca,'Ytick',0:20:60);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[AKG]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 60]);
text((500-0)*0.05,(60-0)*0.9,'C','FontSize',24);

% [SUC]
subplot(3,2,4);
box on; hold on;
plot(t((length(t1)+1):end),(SUC_mito((length(t1)+1):end)+SUC_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, SUC_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',14);
set(gca,'Ytick',0:10:30);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[SUC]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 30]);
text((500-0)*0.05,(30-0)*0.9,'D','FontSize',24);

% [MAL]
subplot(3,2,5);
box on; hold on;
plot(t((length(t1)+1):end),(MAL_mito((length(t1)+1):end)+MAL_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, MAL_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',14);
set(gca,'Ytick',0:500:2000);
set(gca,'Xtick',0:100:500);
% set(gca,'XtickLabel',[]);
xlabel('Time (sec)');
ylabel('[MAL]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 2000]);
text((500-0)*0.05,(2000-0)*0.9,'E','FontSize',24);


%% Part II: Simulating LaNoue et al.'s experiments with 1mM pyruvate
x0 = TCA_x0_vitro( modelInfo );

%% Add PI for incubation
x0 = setParameter( x0, modelInfo.SVarID, 'H_cytoplasm', 10^(-7.2) );   % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'M_cytoplasm', 5e-3 );        % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'K_cytoplasm', 150e-3 );      % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'Pi_cytoplasm', 20e-3 );      % LaNoue et al., 1970
x0 = setParameter( x0, modelInfo.SVarID, 'ADP_cytoplasm', 1e-12 );     % LaNoue et al., 1970

%% no glucose in the cytoplasm
x0 = setParameter( x0, modelInfo.SVarID, 'glucose_cytoplasm', 30e-12 ); 

%% load 5 mM asparatate into the matrix
x0 = setParameter( x0, modelInfo.SVarID, 'aspartate_cytoplasm', 1e-12 );
x0 = setParameter( x0, modelInfo.SVarID, 'aspartate_matrix', 5e-3 );

%% Incubate
disp( 'Incubating...' );
[t,y] = ode15s(@dXdTLaNoue, [0 t_ib], x0, options, T, BX, K_BX, params );
x1 = y;
t1 = t;

%% Add pyruvate
x0 = y(end, :);
x0 = setParameter( x0, modelInfo.SVarID, 'pyruvate_cytoplasm' ,   1e-3 );

%% Run the experiment for 250 seconds
disp( 'Doing the experiment...' );
[t,y] = ode15s(@dXdTLaNoue, [0 250], x0, options, T, BX, K_BX, params );
x = y;
torig = t;
 
x = [x1' x']';
t = [(t1-t_ib)' t']';

%% Computing simulated concentrations
GLU_mito = ( x(:,modelInfo.SVarID.glutamate_matrix)*W_x + x(:,modelInfo.SVarID.glutamate_im)*W_i )*rho_m; 
GLU_extm = x(:,modelInfo.SVarID.glutamate_cytoplasm)*W_c*rho_m;

ASP_mito = ( x(:,modelInfo.SVarID.aspartate_matrix)*W_x + x(:,modelInfo.SVarID.aspartate_im)*W_i )*rho_m; 
ASP_extm = x(:,modelInfo.SVarID.aspartate_cytoplasm)*W_c*rho_m;

%% Plotting figures
% [GLU] and [ASP]
subplot(3,2,6);
box on; hold on;
plot(t((length(t1)+1):end),(ASP_mito((length(t1)+1):end)+ASP_extm((length(t1)+1):end))*1e9,'k--','LineWidth',2.0);
p = plot(t_exp_fig3, ASP_exp, 'ko','MarkerFaceColor',[1 1 1],'MarkerSize',12);
plot(t((length(t1)+1):end),(GLU_mito((length(t1)+1):end)+GLU_extm((length(t1)+1):end))*1e9,'k-','LineWidth',2.0);
p = plot(t_exp_fig3, GLU_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',14);
set(gca,'Ytick',0:5:15);
set(gca,'Xtick',0:50:250);
% set(gca,'XtickLabel',[]);
xlabel('Time (sec)');
ylabel('[GLU]_{total} and [ASP]_{total} (nmoles/mg)');
xlim([0 250]);
ylim([0 15]);
text((500-0)*0.05,(15-0)*0.9,'F','FontSize',24);

%% End of simulation
display( 'Done!' );
