% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( A )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [ATP_A , ADP_A , Pi_A ]
%
%
% Free Parameters:
% [k1_ATPASE ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
%% LIST OF STATE VARIABLES
% 1 ATP_A
% 2 ADP_A
% 3 Pi_A
% 4 h_A
% 5 m_A
% 6 k_A

% PARTIAL VOLUME FRACTIONS
VWater_A = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_A = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
ATP_A = x(1);
ADP_A = x(2);
Pi_A = x(3);
% Concentrations of H, Mg, and K
h_A = x(4);
m_A = x(5);
k_A = x(6);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% ATP_A
Kh(1) = 2.7990983755242071e-007;
Km(1) = 0.00010815244062499601;
Kk(1) = 0.097055055483606045;
% ADP_A
Kh(2) = 4.1856568564882887e-007;
Km(2) = 0.00088211913575503352;
Kk(2) = 0.13114858875318428;
% Pi_A
Kh(3) = 2.1306351186738022e-007;
Km(3) = 0.032137949367841569;
Kk(3) = 0.37888645618434613;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_A/Kh(1) + m_A/Km(1) + k_A/Kk(1);
P( 2 ) = 1  + h_A/Kh(2) + m_A/Km(2) + k_A/Kk(2);
P( 3 ) = 1  + h_A/Kh(3) + m_A/Km(3) + k_A/Kk(3);

%% THERMODYNAMIC EQUATIONS
DGro_ATPASE =4.5083;

Keq_ATPASE_A = exp(-DGro_ATPASE/RT)/P(1)*P(2)*P(3)/h_A;

%% FLUX EQUATIONS
%ATPASE_A
k1_ATPASE=par(1);
J_ATPASE_A=k1_ATPASE*(ATP_A-ADP_A*Pi_A/Keq_ATPASE_A);

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_ATPASE_A ) / VWater_A; % ATP_A
f(2,:) = ( 0  + 1*J_ATPASE_A ) / VWater_A; % ADP_A
f(3,:) = ( 0  + 1*J_ATPASE_A ) / VWater_A; % Pi_A

%% ION EQUATIONS
% COMPARTMENT A:
ii = [1  2  3]; % Indices of SVs in compartment A
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_A*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_A*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_A./Km(ii)+k_A./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_A*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_A*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_A./Kh(ii)+k_A./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_A*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_A*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_A./Kh(ii)+m_A./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 1*J_ATPASE_A) / VWater_A;
J_M = (0) / VWater_A;
J_K = (0) / VWater_A;
Phi_H = J_H - sum( h_A*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_A*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_A*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_A/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(4,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(5,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(6,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_ATPASE_A];
